const { resolve } = require('path')
const Compression = require('compression-webpack-plugin')
const critical = require('critical')
const glob = require('glob')
const PurgecssPlugin = require('purgecss-webpack-plugin')

const srcDir = resolve('./src')

class CustomExtractor {
  static extract(content) {
    return content.match(/[A-Za-z0-9-_:/]+/g) || []
  }
}

module.exports = (api) => {
  api.chainWebpack((config) => {
    if (process.env.NODE_ENV === 'production') {
      // https://markus.oberlehner.net/blog/how-to-pre-render-vue-powered-websites-with-webpack/
      config.plugin('pre-render').tap(args => [
        {
          ...args[0],
          async postProcess(renderedRoute) {
            renderedRoute.html = renderedRoute.html
              .replace(/<script (.*?)>/g, '<script $1 async>')
              .replace('id="app"', 'id="app" data-server-rendered="true"')

            renderedRoute.html = (await critical.generate({
              inline: true,
              html: renderedRoute.html,
              base: 'dist/',
              folder: 'dist/',
              width: 1300,
              height: 900,
            })).toString('utf8')

            return renderedRoute
          },
        },
      ])
    }
    // remove the prefetch plugin
    config.plugins.delete('prefetch')

    // use ~ instead of @ because @ for srcDir is just confusing when npm uses it.
    config.resolve.alias.set('~', srcDir)

    // Entrypoint and Asset size should not be greater than 250kb combined (gziped)
    config.performance
      .maxEntrypointSize(125000) // 125kb
      .maxAssetSize(125000) // 125kb
      .assetFilter(assetFilename => assetFilename.endsWith('.js.gz'))

    // Gzip assets
    config
      .plugin('compress')
      .use(Compression, [
        {
          exclude: /.*.html$/,
        },
      ])
      .end()

    config.plugin('purge-css')
      .use(PurgecssPlugin, [{
        paths: () => glob.sync(`${srcDir}/**/!(*.test)*.{html,vue,css,scss}`, { nodir: true }),
        whitelist: () => ['app', 'html', 'body', 'fa', 'svg', 'animated'],
        whitelistPatterns: () => [
          /[A-Za-z0-9_-]+__[A-Za-z0-9_-]+/,
          /^fa-.*/,
          /^svg-.*/,
          /^animated.*/,
          /^slide.*/,
        ],
        extractors: [
          {
            extractor: CustomExtractor,
            extensions: ['html', 'jsx', 'js', 'tsx', 'ts'],
          },
        ],
      },
      ])

    // Enable graphql files
    config.module
      .rule('graphql')
      .test(/\.graphql|\.gql$/)
      .use('graphql-tag/loader')
      .loader('graphql-tag/loader')

    // FIXME: https://github.com/graphql/graphql-js/issues/1272
    config.module
      .rule('mjs')
      .test(/\.mjs$/)
      .include.add(/node_modules/)
      .end()
      .type('javascript/auto')

    config.resolve.extensions
      .add('.mjs')
      .add('.gql')
      .add('.graphql')
  })

  api.extendPackage({
    devDependencies: {
      'compression-webpack-plugin': '^2.0.0',
      critical: '^1.3.4',
      glob: '^7.1.3',
      'purgecss-webpack-plugin': '^1.3.1',
      'postcss-flexbugs-fixes': '^4.1.0',
      'postcss-preset-env': '^6.1.1',
    },
  })
}
